@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: GtkNSSDialogs.cpp,v 1.41.2.1 2006/04/17 14:22:40 chpe Exp $
+ *  $Id: GtkNSSDialogs.cpp,v 1.41.2.3 2006/07/06 13:32:33 chpe Exp $
  */
 
 /*
@@ -32,6 +32,7 @@
 
 #include "EphyUtils.h"
 #include "AutoJSContextStack.h"
+#include "AutoWindowModalState.h"
 
 #include <nsCOMPtr.h>
 #include <nsMemory.h>
@@ -212,6 +213,9 @@ display_cert_warning_box (nsIInterfaceRequestor *ctx,
 	GtkWidget *dialog, *label, *checkbox, *vbox, *button;
 	int res;
 
+	g_return_val_if_fail (markup_text, GTK_RESPONSE_CANCEL);
+	g_return_val_if_fail (!checkbox_text || checkbox_value, GTK_RESPONSE_CANCEL);
+
 	nsresult rv;
 	AutoJSContextStack stack;
 	rv = stack.Init ();
@@ -220,12 +224,11 @@ display_cert_warning_box (nsIInterfaceRequestor *ctx,
 	/* NOTE: Due to a mozilla bug [https://bugzilla.mozilla.org/show_bug.cgi?id=306288],
 	 * we will always end up without a parent!
 	 */
-	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
+	nsCOMPtr<nsIDOMWindow> parent (do_GetInterface (ctx));
 	GtkWindow *gparent = GTK_WINDOW (EphyUtils::FindGtkParent (parent));
 
-        g_return_val_if_fail (markup_text, GTK_RESPONSE_CANCEL);
-        g_return_val_if_fail (!checkbox_text || checkbox_value, GTK_RESPONSE_CANCEL);
-	
+	AutoWindowModalState modalState (parent);
+
 	dialog = gtk_dialog_new_with_buttons ("", gparent,
 					      GTK_DIALOG_DESTROY_WITH_PARENT,
 					      (char *) NULL);
@@ -554,9 +557,11 @@ GtkNSSDialogs::ConfirmDownloadCACert(nsIInterfaceRequestor *ctx,
 	rv = stack.Init ();
 	if (NS_FAILED (rv)) return rv;
 
-	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
+	nsCOMPtr<nsIDOMWindow> parent (do_GetInterface (ctx));
 	GtkWindow *gparent = GTK_WINDOW (EphyUtils::FindGtkParent (parent));
 
+	AutoWindowModalState modalState (parent);
+
 	dialog = gtk_dialog_new_with_buttons (_("Trust new Certificate Authority?"), gparent,
 					      GTK_DIALOG_DESTROY_WITH_PARENT,
 					      _("_View Certificate"),
@@ -771,9 +776,11 @@ GtkNSSDialogs::SetPKCS12FilePassword(nsIInterfaceRequestor *ctx,
 	rv = stack.Init ();
 	if (NS_FAILED (rv)) return rv;
 
-	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
+	nsCOMPtr<nsIDOMWindow> parent (do_GetInterface (ctx));
 	GtkWindow *gparent = GTK_WINDOW (EphyUtils::FindGtkParent (parent));
 
+	AutoWindowModalState modalState (parent);
+
 	dialog = gtk_dialog_new_with_buttons ("", gparent,
 					      GTK_DIALOG_DESTROY_WITH_PARENT,
 					      GTK_STOCK_CANCEL,
@@ -894,9 +901,11 @@ GtkNSSDialogs::GetPKCS12FilePassword(nsIInterfaceRequestor *ctx,
 	rv = stack.Init ();
 	if (NS_FAILED (rv)) return rv;
 
-	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
+	nsCOMPtr<nsIDOMWindow> parent (do_GetInterface (ctx));
 	GtkWindow *gparent = GTK_WINDOW (EphyUtils::FindGtkParent (parent));
 
+	AutoWindowModalState modalState (parent);
+
 	dialog = gtk_dialog_new_with_buttons ("", gparent,
 					      GTK_DIALOG_DESTROY_WITH_PARENT,
 					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
@@ -1354,8 +1363,11 @@ GtkNSSDialogs::ViewCert(nsIInterfaceRequestor *ctx,
 	dialog = glade_xml_get_widget (gxml, "viewcert_dialog");
 	g_return_val_if_fail (dialog != NULL, NS_ERROR_FAILURE);
 
-	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
+	nsCOMPtr<nsIDOMWindow> parent (do_GetInterface (ctx));
 	GtkWindow *gparent = GTK_WINDOW (EphyUtils::FindGtkParent (parent));
+
+	AutoWindowModalState modalState (parent);
+
 	if (gparent)
 	{
 		gtk_window_set_transient_for (GTK_WINDOW(dialog), GTK_WINDOW(gparent));
