@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: GtkNSSKeyPairDialogs.cpp,v 1.13.4.4 2004/11/24 21:16:10 chpe Exp $
+ *  $Id: GtkNSSKeyPairDialogs.cpp,v 1.26.2.2 2006/07/06 13:32:33 chpe Exp $
  */
 
 /*
@@ -37,27 +37,25 @@
 
 #include "mozilla-config.h"
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
-
-#ifdef HAVE_MOZILLA_PSM
 
 #include "EphyUtils.h"
+#include "AutoJSContextStack.h"
+#include "AutoWindowModalState.h"
 
 #include <nsIServiceManager.h>
 #include <nsIInterfaceRequestor.h>
 #include <nsIInterfaceRequestorUtils.h>
 #include <nsIKeygenThread.h>
-
-#ifdef HAVE_NSIKEYGENTHREAD_NSIOBSERVER
-#include <nsIObserver.h>
-#else /* !HAVE_NSIKEYGENTHREAD_NSIOBSERVER */
 #include <nsIDOMWindow.h>
+
+#ifdef HAVE_GECKO_1_8
+#include <nsIObserver.h>
+#else /* !HAVE_GECKO_1_8 */
 #ifdef ALLOW_PRIVATE_API
-#include "nsIDOMWindowInternal.h"
+#include <nsIDOMWindowInternal.h>
 #endif /* ALLOW_PRIVATE_API */
-#endif /* HAVE_NSIKEYGENTHREAD_NSIOBSERVER */
+#endif /* HAVE_GECKO_1_8 */
 
 #include <gtk/gtkdialog.h>
 #include <gtk/gtkprogressbar.h>
@@ -70,22 +68,23 @@
 #include <glib/gi18n.h>
 
 #include "GtkNSSKeyPairDialogs.h"
+#include "ephy-gui.h"
 #include "ephy-debug.h"
 
 GtkNSSKeyPairDialogs::GtkNSSKeyPairDialogs ()
 {
-	LOG ("GtkNSSKeyPairDialogs ctor (%p)", this)
+	LOG ("GtkNSSKeyPairDialogs ctor (%p)", this);
 }
 
 GtkNSSKeyPairDialogs::~GtkNSSKeyPairDialogs ()
 {
-	LOG ("GtkNSSKeyPairDialogs dtor (%p)", this)
+	LOG ("GtkNSSKeyPairDialogs dtor (%p)", this);
 }
 
-NS_IMPL_ISUPPORTS1 (GtkNSSKeyPairDialogs, 
-		    nsIGeneratingKeypairInfoDialogs)
+NS_IMPL_THREADSAFE_ISUPPORTS1 (GtkNSSKeyPairDialogs,
+			       nsIGeneratingKeypairInfoDialogs)
 
-#ifdef HAVE_NSIKEYGENTHREAD_NSIOBSERVER
+#ifdef HAVE_GECKO_1_8
 
 class KeyPairObserver : public nsIObserver
 {
@@ -108,7 +107,7 @@ NS_IMETHODIMP KeyPairObserver::Observe (nsISupports *aSubject, const char *aTopi
        return NS_OK;
 }
 
-#else /* !HAVE_NSIKEYGENTHREAD_NSIOBSERVER */
+#else /* !HAVE_GECKO_1_8 */
 
 /* ------------------------------------------------------------
  * A dummy implementation of nsIDomWindowInternal so that
@@ -143,7 +142,7 @@ NS_IMETHODIMP KeyPairHelperWindow::Close()
 	return NS_OK;
 }
 
-#endif /* HAVE_NSIKEYGENTHREAD_NSIOBSERVER */
+#endif /* HAVE_GECKO_1_8 */
 
 /* ------------------------------------------------------------ */
 static void
@@ -170,11 +169,11 @@ struct KeyPairInfo
 {
 	GtkWidget *progress;
 	GtkWidget *dialog;
-#ifdef HAVE_NSIKEYGENTHREAD_NSIOBSERVER
+#ifdef HAVE_GECKO_1_8
 	KeyPairObserver *helper;
 #else
  	KeyPairHelperWindow *helper;
-#endif /* HAVE_NSIKEYGENTHREAD_NSIOBSERVER */
+#endif /* HAVE_GECKO_1_8 */
 };
 
 
@@ -200,13 +199,28 @@ GtkNSSKeyPairDialogs::DisplayGeneratingKeypairInfo (nsIInterfaceRequestor *ctx,
 	GtkWidget *dialog, *progress, *label, *vbox;
 	gint timeout_id;
 
+	nsresult rv;
+	AutoJSContextStack stack;
+	rv = stack.Init ();
+	if (NS_FAILED (rv)) return rv;
 
 	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
-	GtkWidget *gparent = EphyUtils::FindGtkParent (parent);
+	GtkWindow *gparent = GTK_WINDOW (EphyUtils::FindGtkParent (parent));
 
-	dialog = gtk_dialog_new_with_buttons ("", GTK_WINDOW (gparent),
-					      GTK_DIALOG_NO_SEPARATOR, NULL);
+	AutoWindowModalState modalState (parent);
 
+	dialog = gtk_dialog_new_with_buttons ("", gparent,
+					      GTK_DIALOG_DESTROY_WITH_PARENT, (char *) NULL);
+
+	if (gparent)
+	{
+		gtk_window_group_add_window (ephy_gui_ensure_window_group (gparent),
+					     GTK_WINDOW (dialog));
+	}
+
+	gtk_window_set_icon_name (GTK_WINDOW (dialog), "web-browser");
+
+	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
 
@@ -232,7 +246,7 @@ GtkNSSKeyPairDialogs::DisplayGeneratingKeypairInfo (nsIInterfaceRequestor *ctx,
 
 	/* Create a helper class that just waits for close events
 	 * from the other thread */
-#ifdef HAVE_NSIKEYGENTHREAD_NSIOBSERVER
+#ifdef HAVE_GECKO_1_8
        nsCOMPtr<KeyPairObserver> helper = new KeyPairObserver;
 #else
 	nsCOMPtr<KeyPairHelperWindow> helper = new KeyPairHelperWindow;
@@ -262,7 +276,7 @@ GtkNSSKeyPairDialogs::DisplayGeneratingKeypairInfo (nsIInterfaceRequestor *ctx,
 	return NS_OK;
 }
 
-#ifndef HAVE_NSIKEYGENTHREAD_NSIOBSERVER
+#ifndef HAVE_GECKO_1_8
 
 /*************************************************************
  * Misc functions for the nsIDomWindowInternal implementation
@@ -750,6 +764,4 @@ NS_IMETHODIMP KeyPairHelperWindow::GetWindowRoot(nsIDOMEventTarget * *aWindowRoo
     MOZ_NOT_IMPLEMENTED
 }
 
-#endif /* !HAVE_NSIKEYGENTHREAD_NSIOBSERVER */
-
-#endif
+#endif /* !HAVE_GECKO_1_8 */
