@@ -1,6 +1,6 @@
 /*
- *  Copyright (C) 2003 Marco Pesenti Gritti
- *  Copyright (C) 2003 Christian Persch
+ *  Copyright (C) 2003, 2004 Marco Pesenti Gritti
+ *  Copyright (C) 2003, 2004 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,31 +16,41 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-location-action.c,v 1.25.4.1 2004/11/01 15:14:20 chpe Exp $
+ *  $Id: ephy-location-action.c,v 1.44.2.2 2006/07/24 11:18:36 chpe Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "ephy-location-action.h"
 #include "ephy-location-entry.h"
 #include "ephy-shell.h"
 #include "ephy-completion-model.h"
+#include "ephy-link.h"
 #include "ephy-debug.h"
 
+#include <gdk/gdkkeysyms.h>
 #include <gtk/gtkentry.h>
+#include <gtk/gtkstock.h>
+#include <gtk/gtkimage.h>
 #include <gtk/gtkentrycompletion.h>
+#include <gtk/gtkmain.h>
 
 #define EPHY_LOCATION_ACTION_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_LOCATION_ACTION, EphyLocationActionPrivate))
 
 struct _EphyLocationActionPrivate
 {
+	EphyWindow *window;
 	GList *actions;
 	char *address;
-	gboolean editable;
+	char *typed_address;
 	EphyNode *smart_bmks;
 	EphyBookmarks *bookmarks;
+	GdkPixbuf *icon;
+	char *lock_stock_id;
+	char *lock_tooltip;
+	guint editable : 1;
+	guint show_lock : 1;
+	guint secure : 1;
 };
 
 static void ephy_location_action_init       (EphyLocationAction *action);
@@ -56,25 +66,30 @@ enum
 {
 	PROP_0,
 	PROP_ADDRESS,
-	PROP_EDITABLE
+	PROP_EDITABLE,
+	PROP_ICON,
+	PROP_LOCK_STOCK,
+	PROP_LOCK_TOOLTIP,
+	PROP_SECURE,
+	PROP_SHOW_LOCK,
+	PROP_WINDOW
 };
 
 enum
 {
-	GO_LOCATION,
+	LOCK_CLICKED,
 	LAST_SIGNAL
 };
+static guint signals[LAST_SIGNAL] = { 0 };
 
 static GObjectClass *parent_class = NULL;
 
-static guint signals[LAST_SIGNAL] = { 0 };
-
 GType
 ephy_location_action_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type)
+	if (G_UNLIKELY (type == 0))
 	{
 		static const GTypeInfo type_info =
 		{
@@ -89,10 +104,11 @@ ephy_location_action_get_type (void)
 			(GInstanceInitFunc) ephy_location_action_init,
 		};
 
-		type = g_type_register_static (GTK_TYPE_ACTION,
+		type = g_type_register_static (EPHY_TYPE_LINK_ACTION,
 					       "EphyLocationAction",
 					       &type_info, 0);
 	}
+
 	return type;
 }
 
@@ -105,6 +121,7 @@ action_activated_cb (GtkEntryCompletion *completion,
 	char *content;
 
 	entry = gtk_entry_completion_get_entry (completion);
+	gtk_im_context_reset (GTK_ENTRY (entry)->im_context);
 	content = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
 	if (content)
 	{
@@ -117,28 +134,38 @@ action_activated_cb (GtkEntryCompletion *completion,
 	                (node, EPHY_NODE_BMK_PROP_LOCATION);
 		g_return_if_fail (smart_url != NULL);
 
-		url = ephy_bookmarks_solve_smart_url
+		url = ephy_bookmarks_resolve_address
 			(action->priv->bookmarks, smart_url, content);
-		g_return_if_fail (url != NULL);
+		g_free (content);
+		if (url == NULL) return;
 
-		g_signal_emit (action, signals[GO_LOCATION], 0, url);
+ 		ephy_link_open (EPHY_LINK (action), url, NULL,
+			        ephy_link_flags_from_current_event ());
 
 		g_free (url);
-		g_free (content);
 	}
 }
 
 static void
-location_url_activate_cb (GtkEntry *entry,
-			  EphyLocationAction *action)
+entry_activate_cb (GtkEntry *entry,
+		   EphyLocationAction *action)
 {
+	EphyBookmarks *bookmarks;
 	const char *content;
+	char *address;
 
 	content = gtk_entry_get_text (entry);
-	if (content)
-	{
-		g_signal_emit (action, signals[GO_LOCATION], 0, content);
-	}
+	if (content == NULL || content[0] == '\0') return;
+
+	bookmarks = ephy_shell_get_bookmarks (ephy_shell_get_default ());
+
+	address = ephy_bookmarks_resolve_address (bookmarks, content, NULL);
+	g_return_if_fail (address != NULL);
+
+	ephy_link_open (EPHY_LINK (action), address, NULL, 
+		        ephy_link_flags_from_current_event ());
+
+	g_free (address);
 }
 
 static void
@@ -148,101 +175,191 @@ user_changed_cb (GtkWidget *proxy, EphyLocationAction *action)
 
 	address = ephy_location_entry_get_location (EPHY_LOCATION_ENTRY (proxy));
 
-	LOG ("user_changed_cb, new address %s", address)
+	LOG ("user_changed_cb, new address %s", address);
 
 	g_signal_handlers_block_by_func (action, G_CALLBACK (sync_address), proxy);
-	ephy_location_action_set_address (action, address);
+	ephy_location_action_set_address (action, address, NULL);
 	g_signal_handlers_unblock_by_func (action, G_CALLBACK (sync_address), proxy);
 }
 
 static void
-sync_address (GtkAction *act, GParamSpec *pspec, GtkWidget *proxy)
+lock_clicked_cb (GtkWidget *proxy,
+		 EphyLocationAction *action)
 {
-	EphyLocationAction *action = EPHY_LOCATION_ACTION (act);
+	g_signal_emit (action, signals[LOCK_CLICKED], 0);
+}
 
-	LOG ("sync_address")
+static void
+sync_address (GtkAction *gaction,
+	      GParamSpec *pspec,
+	      GtkWidget *proxy)
+{
+	EphyLocationAction *action = EPHY_LOCATION_ACTION (gaction);
+	EphyLocationActionPrivate *priv = action->priv;
+	EphyLocationEntry *lentry = EPHY_LOCATION_ENTRY (proxy);
+
+	LOG ("sync_address %s", action->priv->address);
 
-	g_return_if_fail (EPHY_IS_LOCATION_ENTRY (proxy));
 	g_signal_handlers_block_by_func (proxy, G_CALLBACK (user_changed_cb), action);
-	ephy_location_entry_set_location (EPHY_LOCATION_ENTRY (proxy),
-					  action->priv->address);
+	ephy_location_entry_set_location (lentry, priv->address,
+					  priv->typed_address);
 	g_signal_handlers_unblock_by_func (proxy, G_CALLBACK (user_changed_cb), action);
 }
 
 static void
-sync_editable (GtkAction *act, GParamSpec *pspec, GtkWidget *proxy)
+sync_editable (GtkAction *gaction,
+	       GParamSpec *pspec,
+	       GtkWidget *proxy)
 {
-	EphyLocationAction *action = EPHY_LOCATION_ACTION (act);
+	EphyLocationAction *action = EPHY_LOCATION_ACTION (gaction);
+	EphyLocationEntry *lentry = EPHY_LOCATION_ENTRY (proxy);
 	GtkWidget *entry;
 
-	entry = GTK_BIN (proxy)->child;
+	entry = ephy_location_entry_get_entry (lentry);
 	gtk_editable_set_editable (GTK_EDITABLE (entry), action->priv->editable);
 }
 
 static void
-remove_completion_actions (GtkAction *action, GtkWidget *proxy)
+sync_icon (GtkAction *gaction,
+	   GParamSpec *pspec,
+	   GtkWidget *proxy)
 {
+	EphyLocationAction *action = EPHY_LOCATION_ACTION (gaction);
+	EphyLocationActionPrivate *priv = action->priv;
+	EphyLocationEntry *entry = EPHY_LOCATION_ENTRY (proxy);
+
+	ephy_location_entry_set_favicon (entry, priv->icon);
+}
+
+static void
+sync_lock_stock_id (GtkAction *gaction,
+		    GParamSpec *pspec,
+		    GtkWidget *proxy)
+{
+	EphyLocationAction *action = EPHY_LOCATION_ACTION (gaction);
+	EphyLocationActionPrivate *priv = action->priv;
+	EphyLocationEntry *entry = EPHY_LOCATION_ENTRY (proxy);
+
+	ephy_location_entry_set_lock_stock (entry, priv->lock_stock_id);
+}
+
+static void
+sync_lock_tooltip (GtkAction *gaction,
+		   GParamSpec *pspec,
+		   GtkWidget *proxy)
+{
+	EphyLocationAction *action = EPHY_LOCATION_ACTION (gaction);
+	EphyLocationActionPrivate *priv = action->priv;
+	EphyLocationEntry *entry = EPHY_LOCATION_ENTRY (proxy);
+
+	ephy_location_entry_set_lock_tooltip (entry, priv->lock_tooltip);
+}
+
+static void
+sync_secure (GtkAction *gaction,
+	     GParamSpec *pspec,
+	     GtkWidget *proxy)
+{
+	EphyLocationAction *action = EPHY_LOCATION_ACTION (gaction);
+	EphyLocationActionPrivate *priv = action->priv;
+	EphyLocationEntry *entry = EPHY_LOCATION_ENTRY (proxy);
+
+	ephy_location_entry_set_secure (entry, priv->secure);
+}
+
+static void
+sync_show_lock (GtkAction *gaction,
+	        GParamSpec *pspec,
+	        GtkWidget *proxy)
+{
+	EphyLocationAction *action = EPHY_LOCATION_ACTION (gaction);
+	EphyLocationActionPrivate *priv = action->priv;
+	EphyLocationEntry *entry = EPHY_LOCATION_ENTRY (proxy);
+
+	ephy_location_entry_set_show_lock (entry, priv->show_lock);
+}
+
+static char *
+get_location_cb (EphyLocationEntry *entry,
+		EphyLocationAction *action)
+{
+	EphyLocationActionPrivate *priv = action->priv;
+	EphyTab *tab;
+	
+	tab = ephy_window_get_active_tab (priv->window);
+	g_return_val_if_fail (tab != NULL, NULL);
+
+	return g_strdup (ephy_tab_get_address (tab));
+}
+
+static char *
+get_title_cb (EphyLocationEntry *entry,
+	      EphyLocationAction *action)
+{
+	EphyTab *tab;
+
+	tab = ephy_window_get_active_tab (action->priv->window);
+
+	return g_strdup (ephy_tab_get_title (tab));
+}
+
+static void
+remove_completion_actions (GtkAction *gaction,
+			   GtkWidget *proxy)
+{
+	EphyLocationAction *action = EPHY_LOCATION_ACTION (gaction);
+	EphyLocationEntry *lentry = EPHY_LOCATION_ENTRY (proxy);
 	GtkWidget *entry;
 	GtkEntryCompletion *completion;
-	EphyLocationAction *la = EPHY_LOCATION_ACTION (action);
 	GList *l;
 
-	entry = GTK_BIN (proxy)->child;
+	entry = ephy_location_entry_get_entry (lentry);
 	completion = gtk_entry_get_completion (GTK_ENTRY (entry));
 
-	for (l = la->priv->actions; l != NULL; l = l->next)
+	for (l = action->priv->actions; l != NULL; l = l->next)
 	{
 		gtk_entry_completion_delete_action (completion, 0);
 	}
 
 	g_signal_handlers_disconnect_by_func
-			(completion, G_CALLBACK (action_activated_cb), la);
+			(completion, G_CALLBACK (action_activated_cb), action);
 }
 
 static void
-add_completion_actions (GtkAction *action, GtkWidget *proxy)
+add_completion_actions (GtkAction *gaction,
+			GtkWidget *proxy)
 {
+	EphyLocationAction *action = EPHY_LOCATION_ACTION (gaction);
+	EphyLocationEntry *lentry = EPHY_LOCATION_ENTRY (proxy);
 	GtkWidget *entry;
 	GtkEntryCompletion *completion;
-	EphyLocationAction *la = EPHY_LOCATION_ACTION (action);
 	GList *l;
 
-	entry = GTK_BIN (proxy)->child;
+	entry = ephy_location_entry_get_entry (lentry);
 	completion = gtk_entry_get_completion (GTK_ENTRY (entry));
 
-	for (l = la->priv->actions; l != NULL; l = l->next)
+	for (l = action->priv->actions; l != NULL; l = l->next)
 	{
 		EphyNode *bmk = l->data;
 		const char *title;
 		int index;
 
-		index = g_list_position (la->priv->actions, l);
+		index = g_list_position (action->priv->actions, l);
 		title = ephy_node_get_property_string
 	                (bmk, EPHY_NODE_BMK_PROP_TITLE);
 		gtk_entry_completion_insert_action_text (completion, index, (char*)title);
 	}
 
 	g_signal_connect (completion, "action_activated",
-			  G_CALLBACK (action_activated_cb), la);
-}
-
-static gboolean
-create_menu_proxy_cb (GtkToolItem *tool_item, 
-		      GtkAction *action)
-{
-	gtk_tool_item_set_proxy_menu_item
-		(tool_item, "ephy-location-action-menu-item", NULL);
-
-	return TRUE;
+			  G_CALLBACK (action_activated_cb), action);
 }
 
 static void
 connect_proxy (GtkAction *action, GtkWidget *proxy)
 {
-	LOG ("Connect proxy")
-
 	if (EPHY_IS_LOCATION_ENTRY (proxy))
 	{
+		EphyLocationEntry *lentry = EPHY_LOCATION_ENTRY (proxy);
 		EphyCompletionModel *model;
 		GtkWidget *entry;
 
@@ -262,51 +379,58 @@ connect_proxy (GtkAction *action, GtkWidget *proxy)
 		sync_editable (action, NULL, proxy);
 		g_signal_connect_object (action, "notify::editable",
 					 G_CALLBACK (sync_editable), proxy, 0);
+		sync_icon (action, NULL, proxy);
+		g_signal_connect_object (action, "notify::icon",
+					 G_CALLBACK (sync_icon), proxy, 0);
+		sync_lock_stock_id (action, NULL, proxy);
+		g_signal_connect_object (action, "notify::lock-stock-id",
+					 G_CALLBACK (sync_lock_stock_id), proxy, 0);
+		sync_lock_tooltip (action, NULL, proxy);
+		g_signal_connect_object (action, "notify::lock-tooltip",
+					 G_CALLBACK (sync_lock_tooltip), proxy, 0);
+		sync_secure (action, NULL, proxy);
+		g_signal_connect_object (action, "notify::secure",
+					 G_CALLBACK (sync_secure), proxy, 0);
+		sync_show_lock (action, NULL, proxy);
+		g_signal_connect_object (action, "notify::show-lock",
+					 G_CALLBACK (sync_show_lock), proxy, 0);
 
-		entry = GTK_BIN (proxy)->child;
+		entry = ephy_location_entry_get_entry (lentry);
 		g_signal_connect_object (entry, "activate",
-					 G_CALLBACK (location_url_activate_cb),
+					 G_CALLBACK (entry_activate_cb),
 					 action, 0);
-		g_signal_connect_object (proxy, "user_changed",
+		g_signal_connect_object (proxy, "user-changed",
 					 G_CALLBACK (user_changed_cb), action, 0);
+		g_signal_connect_object (proxy, "lock-clicked",
+					 G_CALLBACK (lock_clicked_cb), action, 0);
+		g_signal_connect_object (proxy, "get-location",
+					 G_CALLBACK (get_location_cb), action, 0);
+		g_signal_connect_object (proxy, "get-title",
+					 G_CALLBACK (get_title_cb), action, 0);
 	}
 
-	/* FIXME: when porting to gtk+ 2.6, use "visible-overflown" 
-	 * property on GtkAction instead.
-	 * we need to connect to this before chaining up, since gtkaction's
-	 * connect_proxy connects a routine there which uses create_menu_item
-	 * method to generate a menu proxy (and create_menu_item CANNOT return
-	 * NULL. See bug #133446.
-	 */
-	g_signal_connect_object (proxy, "create_menu_proxy",
-				 G_CALLBACK (create_menu_proxy_cb),
-				 action, 0);
-
-	(* GTK_ACTION_CLASS (parent_class)->connect_proxy) (action, proxy);
+	GTK_ACTION_CLASS (parent_class)->connect_proxy (action, proxy);
 }
 
 static void
 disconnect_proxy (GtkAction *action, GtkWidget *proxy)
 {
-	LOG ("Disconnect proxy")
+	GTK_ACTION_CLASS (parent_class)->disconnect_proxy (action, proxy);
 
 	if (EPHY_IS_LOCATION_ENTRY (proxy))
 	{
+		EphyLocationEntry *lentry = EPHY_LOCATION_ENTRY (proxy);
 		GtkWidget *entry;
 
-		entry = GTK_BIN (proxy)->child;
+		entry = ephy_location_entry_get_entry (lentry);
 
-		g_signal_handlers_disconnect_by_func
-			(action, G_CALLBACK (sync_address), proxy);
-
-		g_signal_handlers_disconnect_by_func
-			(entry, G_CALLBACK (location_url_activate_cb), action);
-
-		g_signal_handlers_disconnect_by_func
-			(proxy, G_CALLBACK (user_changed_cb), action);
+		g_signal_handlers_disconnect_matched (action, G_SIGNAL_MATCH_DATA,
+						      0, 0, NULL, NULL, proxy);
+		g_signal_handlers_disconnect_matched (proxy, G_SIGNAL_MATCH_DATA,
+						      0, 0, NULL, NULL, action);
+		g_signal_handlers_disconnect_matched (entry, G_SIGNAL_MATCH_DATA,
+						      0, 0, NULL, NULL, action);
 	}
-
-	(* GTK_ACTION_CLASS (parent_class)->disconnect_proxy) (action, proxy);
 }
 
 static void
@@ -316,14 +440,39 @@ ephy_location_action_set_property (GObject *object,
 				   GParamSpec *pspec)
 {
 	EphyLocationAction *action = EPHY_LOCATION_ACTION (object);
+	EphyLocationActionPrivate *priv = action->priv;
 
 	switch (prop_id)
 	{
 		case PROP_ADDRESS:
-			ephy_location_action_set_address (action, g_value_get_string (value));
+			ephy_location_action_set_address (action, g_value_get_string (value), NULL);
 			break;
 		case PROP_EDITABLE:
-			action->priv->editable = g_value_get_boolean (value);
+			priv->editable = g_value_get_boolean (value);
+			break;
+		case PROP_ICON:
+			if (priv->icon != NULL)
+			{
+				g_object_unref (priv->icon);
+			}
+			priv->icon = GDK_PIXBUF (g_value_dup_object (value));
+			break;
+		case PROP_LOCK_STOCK:
+			g_free (priv->lock_stock_id);
+			priv->lock_stock_id = g_value_dup_string (value);
+			break;
+		case PROP_LOCK_TOOLTIP:
+			g_free (priv->lock_tooltip);
+			priv->lock_tooltip = g_value_dup_string (value);
+			break;
+		case PROP_SECURE:
+			priv->secure = g_value_get_boolean (value);
+			break;
+		case PROP_SHOW_LOCK:
+			priv->show_lock = g_value_get_boolean (value);
+			break;
+		case PROP_WINDOW:
+			priv->window = EPHY_WINDOW (g_value_get_object (value));
 			break;
 	}
 }
@@ -335,6 +484,7 @@ ephy_location_action_get_property (GObject *object,
 				   GParamSpec *pspec)
 {
 	EphyLocationAction *action = EPHY_LOCATION_ACTION (object);
+	EphyLocationActionPrivate *priv = action->priv;
 
 	switch (prop_id)
 	{
@@ -342,7 +492,25 @@ ephy_location_action_get_property (GObject *object,
 			g_value_set_string (value, ephy_location_action_get_address (action));
 			break;
 		case PROP_EDITABLE:
-			g_value_set_boolean (value, action->priv->editable);
+			g_value_set_boolean (value, priv->editable);
+			break;
+		case PROP_ICON:
+			g_value_set_object (value, priv->icon);
+			break;
+		case PROP_LOCK_STOCK:
+			g_value_set_string (value, priv->lock_stock_id);
+			break;
+		case PROP_LOCK_TOOLTIP:
+			g_value_set_string (value, priv->lock_tooltip);
+			break;
+		case PROP_SECURE:
+			g_value_set_boolean (value, priv->secure);
+			break;
+		case PROP_SHOW_LOCK:
+			g_value_set_boolean (value, priv->show_lock);
+			break;
+		case PROP_WINDOW:
+			/* not readable */
 			break;
 	}
 }
@@ -363,16 +531,15 @@ ephy_location_action_class_init (EphyLocationActionClass *class)
 	action_class->connect_proxy = connect_proxy;
 	action_class->disconnect_proxy = disconnect_proxy;
 
-	signals[GO_LOCATION] =
-                g_signal_new ("go_location",
-                              G_OBJECT_CLASS_TYPE (object_class),
-                              G_SIGNAL_RUN_FIRST,
-                              G_STRUCT_OFFSET (EphyLocationActionClass, go_location),
-                              NULL, NULL,
-                              g_cclosure_marshal_VOID__STRING,
-                              G_TYPE_NONE,
-                              1,
-			      G_TYPE_STRING);
+	signals[LOCK_CLICKED] = g_signal_new (
+		"lock-clicked",
+		EPHY_TYPE_LOCATION_ACTION,
+		G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (EphyLocationActionClass, lock_clicked),
+		NULL, NULL,
+		g_cclosure_marshal_VOID__VOID,
+		G_TYPE_NONE,
+		0);
 
 	g_object_class_install_property (object_class,
 					 PROP_ADDRESS,
@@ -380,15 +547,63 @@ ephy_location_action_class_init (EphyLocationActionClass *class)
 							      "Address",
 							      "The address",
 							      "",
-							      G_PARAM_READWRITE));
+							      G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 	g_object_class_install_property (object_class,
 					 PROP_EDITABLE,
 					 g_param_spec_boolean ("editable",
 							       "Editable",
 							       "Editable",
 							       TRUE,
-							       G_PARAM_READWRITE));
+							       G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
+	g_object_class_install_property (object_class,
+					 PROP_ICON,
+					 g_param_spec_object ("icon",
+							      "Icon",
+							      "The icon",
+							      GDK_TYPE_PIXBUF,
+							      G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+
+	g_object_class_install_property (object_class,
+					 PROP_LOCK_STOCK,
+					 g_param_spec_string  ("lock-stock-id",
+							       "Lock Stock ID",
+							       "Lock Stock ID",
+							       NULL,
+							       G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+
+	g_object_class_install_property (object_class,
+					 PROP_LOCK_TOOLTIP,
+					 g_param_spec_string  ("lock-tooltip",
+							       "Lock Tooltip",
+							       "The icon",
+							       NULL,
+							       G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+
+	g_object_class_install_property (object_class,
+					 PROP_SECURE,
+					 g_param_spec_boolean ("secure",
+							       "Secure",
+							       "Secure",
+							       FALSE,
+							       G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+
+	g_object_class_install_property (object_class,
+					 PROP_SHOW_LOCK,
+					 g_param_spec_boolean ("show-lock",
+							       "Show Lock",
+							       "Show Lock",
+							       FALSE,
+							       G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+
+	g_object_class_install_property (object_class,
+					 PROP_WINDOW,
+					 g_param_spec_object ("window",
+							      "Window",
+							      "The navigation window",
+							      G_TYPE_OBJECT,
+							      G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB |
+							      G_PARAM_CONSTRUCT_ONLY));
 
 	g_type_class_add_private (object_class, sizeof (EphyLocationActionPrivate));
 }
@@ -500,26 +715,27 @@ actions_child_changed_cb (EphyNode *node,
 static void
 ephy_location_action_init (EphyLocationAction *action)
 {
-	action->priv = EPHY_LOCATION_ACTION_GET_PRIVATE (action);
+	EphyLocationActionPrivate *priv;
 
-	action->priv->address = g_strdup ("");
-	action->priv->actions = NULL;
-	action->priv->editable = TRUE;
-	action->priv->bookmarks = ephy_shell_get_bookmarks (ephy_shell);
-	action->priv->smart_bmks = ephy_bookmarks_get_smart_bookmarks
+	priv = action->priv = EPHY_LOCATION_ACTION_GET_PRIVATE (action);
+
+	priv->address = g_strdup ("");
+	priv->editable = TRUE;
+	priv->bookmarks = ephy_shell_get_bookmarks (ephy_shell);
+	priv->smart_bmks = ephy_bookmarks_get_smart_bookmarks
 		(action->priv->bookmarks);
 
 	init_actions_list (action);
 
-	ephy_node_signal_connect_object (action->priv->smart_bmks,
+	ephy_node_signal_connect_object (priv->smart_bmks,
 			                 EPHY_NODE_CHILD_ADDED,
 			                 (EphyNodeCallback)actions_child_added_cb,
 			                 G_OBJECT (action));
-	ephy_node_signal_connect_object (action->priv->smart_bmks,
+	ephy_node_signal_connect_object (priv->smart_bmks,
 			                 EPHY_NODE_CHILD_REMOVED,
 			                 (EphyNodeCallback)actions_child_removed_cb,
 			                 G_OBJECT (action));
-	ephy_node_signal_connect_object (action->priv->smart_bmks,
+	ephy_node_signal_connect_object (priv->smart_bmks,
 			                 EPHY_NODE_CHILD_CHANGED,
 			                 (EphyNodeCallback)actions_child_changed_cb,
 			                 G_OBJECT (action));
@@ -529,10 +745,18 @@ static void
 ephy_location_action_finalize (GObject *object)
 {
 	EphyLocationAction *action = EPHY_LOCATION_ACTION (object);
+	EphyLocationActionPrivate *priv = action->priv;
 
-	g_list_free (action->priv->actions);
+	if (priv->icon != NULL)
+	{
+		g_object_unref (priv->icon);
+	}
 
-	g_free (action->priv->address);
+	g_list_free (priv->actions);
+	g_free (priv->address);
+	g_free (priv->typed_address);
+	g_free (priv->lock_stock_id);
+	g_free (priv->lock_tooltip);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -547,13 +771,22 @@ ephy_location_action_get_address (EphyLocationAction *action)
 
 void
 ephy_location_action_set_address (EphyLocationAction *action,
-				  const char *address)
+				  const char *address,
+				  const char *typed_address)
 {
+	EphyLocationActionPrivate *priv;
+
 	g_return_if_fail (EPHY_IS_LOCATION_ACTION (action));
 
-	LOG ("set_address %s", address)
+	priv = action->priv;
+
+	LOG ("set_address %s", address);
+
+	g_free (priv->address);
+	priv->address = g_strdup (address);
+
+	g_free (priv->typed_address);
+	priv->typed_address = typed_address ? g_strdup (typed_address) : NULL;
 
-	g_free (action->priv->address);
-	action->priv->address = g_strdup (address);
 	g_object_notify (G_OBJECT (action), "address");
 }
